﻿using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Algorithms.PushGP.Expressions;
using HeuristicLab.Random;

namespace HeuristicLab.Algorithms.PushGP.Generators
{
    public static class CodeGenerator
    {
        private static FastRandom random = new FastRandom();
        public static IEnumerable<Expression> RandomCode(int maxPoints)
        {
            var actualPoints = random.Next(1, maxPoints);

            return RandomCodeWithSize(actualPoints).ToArray();
        }

        private static IEnumerable<Expression> RandomCodeWithSize(int points)
        {
            if (points == 1)
            {
                // TODO: If this is an "ephemeral random constant"???? then return a randomly - chosen value of the appropriate type;
                var opCode = (OpCode)random.Next(OpCodeExtensions.Min, OpCodeExtensions.Max);

                return new[] { ExpressionFactory.Create(opCode) };
            }
            else
            {
                var sizesThisLevel = Decompose(points - 1, points - 1);

                return sizesThisLevel.SelectMany(size => RandomCodeWithSize(size)).Shuffle(random);
            }
        }

        private static IEnumerable<int> Decompose(int number, int maxParts)
        {
            if (number == 1 || maxParts == 1)
            {
                return new[] { number };
            }
            else
            {
                var thisPart = random.Next(1, number - 1);

                return new[] { thisPart }.Concat(Decompose(number - thisPart, maxParts - 1));
            }
        }
    }
}
